/**
 * Component description
 * @moduleName Movable
 * @snippet Movable.snippet
Movable({
    velocity: new Vector2(0, 0)
})
 */
bento.define('components/movable', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    return function (settings) {
        var entity;

        // --- PARAMETERS ---
        var onCollision = settings.onCollision || function () {};
        var stopOnCollide = Utils.isDefined(settings.stopOnCollide) ? settings.stopOnCollide : true;
        var doCollide = Utils.isDefined(settings.doCollide) ? settings.doCollide : true;
        var ignore = settings.ignore || [];
        var enabled = Utils.isDefined(settings.enabled) ? settings.enabled : true;
        var canCollideWith = settings.canCollideWith || ['solids'];
        var checkInvisible = settings.checkInvisible || false;

        // --- COMPONENT ---
        var component = {
            name: 'movableBehaviour',
            velocity: settings.velocity || new Vector2(0, 0),
            lastVelocity: new Vector2(0, 0),
            update: function (data) {
                //are we active?
                if (!enabled) {
                    return;
                }

                // build 'solids' list
                var solids = [];
                Utils.forEach(component.canCollideWith, function (family, i, l, breakLoop) {
                    var list = Bento.objects.getByFamily(family);
                    Utils.forEach(list, function (item, ii, ll, breakLoop2) {
                        var safe = true;

                        //ignore invisible
                        if (!item || (!checkInvisible && !item.visible)) {
                            safe = false;
                        }

                        //check if it's in our ignore list
                        if (safe) {
                            Utils.forEach(component.ignore, function (ignoreItem, i, l, breakLoop) {
                                if (typeof ignoreItem === "string") {
                                    if (item.family.indexOf(ignoreItem) !== -1) {
                                        safe = false;
                                        breakLoop();
                                    }
                                } else {
                                    if (ignoreItem && ignoreItem.id && item.id === ignoreItem.id) {
                                        safe = false;
                                        breakLoop();
                                    }
                                }
                            });
                        }

                        // is it ME?!?!
                        if (safe && item.id === entity.id) {
                            safe = false;
                        }

                        // if we're golden, push it
                        if (safe) {
                            solids.push(item);
                        }
                    });
                });

                //store lastVelocity before we act
                component.lastVelocity = component.velocity.clone();

                //reset stuff
                var didCollide = false;
                var collisionData = {
                    entities: [],
                    velocity: component.velocity.clone(),
                    moved: new Vector2(0, 0),
                    wasX: false,
                    wasY: false
                };

                // function to push us out of 'solid' objects
                var pushOut = function (onPushOut) {
                    //util vars
                    var deltaPos = new Vector2(0, 0);
                    var doPushOut = false;

                    // if we even collide
                    if (doCollide) {
                        //check against our premade list
                        entity.collidesWith({
                            entities: solids,
                            firstOnly: false,
                            onCollide: function (other) {
                                //get bounding Boxes
                                var BB = entity.getBoundingBox();
                                var otherBB = other.getBoundingBox();
                                //get distances to push out of shape
                                var deltaX = (BB.getCenter().x > otherBB.getCenter().x) ? (BB.x - otherBB.getX2()) : (BB.getX2() - otherBB.x);
                                var deltaY = (BB.getCenter().y > otherBB.getCenter().y) ? (BB.y - otherBB.getY2()) : (BB.getY2() - otherBB.y);
                                //grab smallest distance to move out
                                var thisDelta = (Math.abs(deltaX) > Math.abs(deltaY)) ? new Vector2(0, deltaY) : new Vector2(deltaX, 0);
                                // add it to the toal vector for pushing out
                                deltaPos.addTo(thisDelta);
                                //mark that we collided with an entity
                                collisionData.entities.push(other);

                                //mark that we need to move out
                                doPushOut = true;
                            }
                        });

                        if (doPushOut) {
                            // push out in X vector
                            entity.position.subtractFrom(deltaPos);
                            //mark how much we moved
                            collisionData.moved.addTo(deltaPos);
                            didCollide = true;

                            //fire onPushOut
                            if (onPushOut) {
                                onPushOut(collisionData);
                            }
                        }
                    }
                };

                // functions for moving specific axis
                var moveX = function () {
                    // move by X amount
                    entity.position.x += component.velocity.x * data.speed;
                    // push us out of solids we just moved into
                    pushOut(function () {
                        //if we did collide, mak that we did
                        collisionData.wasX = true;
                        // stop us if necessary
                        if (stopOnCollide) {
                            component.velocity.x = 0;
                        }
                    });
                };
                var moveY = function () {
                    // move by X amount
                    entity.position.y += component.velocity.y * data.speed;
                    // push us out of solids we just moved into
                    pushOut(function () {
                        //if we did collide, mak that we did
                        collisionData.wasY = true;
                        // stop us if necessary
                        if (stopOnCollide) {
                            component.velocity.y = 0;
                        }
                    });
                };

                // collision priority is given to the largest vector
                if (Math.abs(component.velocity.x) > Math.abs(component.velocity.y)) {
                    moveX();
                    moveY();
                } else {
                    moveY();
                    moveX();
                }

                // did we collide? yes? send collision data.
                if (didCollide) {
                    onCollision(collisionData);
                }
            },
            ignore: ignore.slice(),
            canCollideWith: canCollideWith.slice(),
            setEnabled: function (newEnabled) {
                enabled = newEnabled;
            },
            attached: function (data) {
                entity = data.entity;
            }
        };
        return component;
    };
});