/**
 * HTML elements helper module
 * @moduleName Html
 * @snippet Html|constructor
Html({})
 */
bento.define('debug/html', [
    'bento',
    'bento/utils'
], function (
    Bento,
    Utils
) {
    'use strict';
    var functions = {
        /**
         * Mix these functions into Html elements
         */
        append: function (child) {
            var i;
            var children;
            var parent = this;
            if (child.length) {
                children = child;
            } else {
                children = arguments;
            }
            // append to array of children to a parent
            for (i = 0; i < children.length; ++i) {
                this.appendChild(children[i]);
            }

            // append returns the parent instead of child!
            return parent;
        },
        write: function (inner) {
            this.innerHTML = inner;
            return this;
        },
        addClass: function (className) {
            var element = this;
            var classNames = element.className.split(' ');
            if (classNames.indexOf(className) === -1) {
                element.className += ' ' + className;
            }
        },
        removeClass: function (className) {
            var element = this;
            var classNames = element.className.split(' ');
            var index = classNames.indexOf(className);
            if (index >= 0) {
                classNames.splice(index, 1);
                element.className = classNames.join(' ');
            }
        },
        removeSelf: function () {
            if (this.parentNode) {
                this.parentNode.removeChild(this);
            }
        }
    };
    /**
     * @snippet Html()|SimpleElement
    Html('$1')
     * @snippet Html()|HtmlElement
Html('$1', {
    id: '$2',
    class: '$3'
})
     */
    var Html = function (elementName, attr) {
        var settings = attr || {};
        var element;
        var children = settings.children;
        var className = settings.class;
        var id = settings.id;
        var style = '';
        var doc;
        delete settings.children;
        delete settings.class;
        delete settings.id;

        if (settings.style && Utils.isObject(settings.style)) {
            // expand into string
            Utils.forEach(settings.style, function (value, name) {
                style += name + ': ' + value + '; ';
            });
            settings.style = style;
        }
        if (elementName.indexOf('<') === 0) {
            // parse from string
            doc = new window.DOMParser().parseFromString(elementName, 'text/html');
            element = doc.body.firstChild;
            // append additional functions and attributes
            Html.addFunctions(element);
            Html.addAttributes(element, settings);
            if (className) {
                element.addClass(className);
            }
            if (id) {
                element.id = id;
            }
        } else {
            // create from settings
            element = Html.make(elementName, className, id, settings);
        }


        if (children) {
            Utils.forEach(children, function (child) {
                element.append(child);
            });
        }

        return element;
    };

    Html.addFunctions = function (element) {
        if (element.addClass) {
            return;
        }
        element.append = functions.append;
        element.attach = functions.append;
        element.write = functions.write;
        element.addClass = functions.addClass;
        element.removeClass = functions.removeClass;
        element.removeSelf = functions.removeSelf;
        Object.defineProperty(element, 'innerHtml', {
            get: function () {
                return this.innerHTML;
            },
            set: function (value) {
                this.innerHTML = value;
            }
        });
    };
    /**
     * @snippet Html.getById()|Element
    Html.getById('$1')
     */
    Html.getById = function (id) {
        var element = document.getElementById(id);
        if (element) {
            Html.addFunctions(element);
        }
        return element;
    };
    Html.appendToBody = function (element) {
        document.body.appendChild(element);
        return element;
    };
    Html.makeDiv = function (className, id, attributes) {
        return this.make('div', className, id, attributes);
    };
    Html.make = function (type, className, id, attributes) {
        var element = document.createElement(type);
        if (id) {
            element.id = id;
        }
        if (className) {
            element.className = className;
        }
        if (attributes) {
            Html.addAttributes(element, attributes);
        }
        Html.addFunctions(element);
        return element;
    };
    Html.addAttributes = function (element, attributes) {
        var attribute;
        for (attribute in attributes) {
            if (!attributes.hasOwnProperty(attribute)) {
                continue;
            }
            if (attribute.indexOf('on') === 0) {
                element[attribute] = attributes[attribute];
            } else if (attribute === 'innerHtml' || attribute === 'innerHTML') {
                element.innerHTML = attributes[attribute];
            } else {
                element.setAttribute(attribute, attributes[attribute]);
            }
        }
    };
    Html.remove = function (element) {
        element.parentNode.removeChild(element);
    };
    /**
     * @snippet Html.removeById()|snippet
    Html.removeById($1)
     */
    Html.removeById = function (id) {
        var element = Html.getById(id);
        if (element) {
            Html.remove(element);
        }
        return element;
    };

    return Html;
});