/**
 * Main module for testing and debugging. Spawn this object and attach to Bento.
 * @moduleName DebugOverlay
 * @snippet DebugOverlay|constructor
DebugOverlay({
    params: {
        'debug/tests/tenjin': null,
        'debug/tests/ironsrc': null,
        'debug/tests/iap': {
            testProduct: '${1:PRODUCT_ID}'
        },
        'debug/tests/facebook': null,
        'debug/tests/systemdata': null,
        'debug/tests/gameanalytics': null,
    }
})
 */
bento.define('debug/overlay', [
    'bento',
    'bento/utils',
    'debug/html',
    'debug/html/accordion'
], function (
    Bento,
    Utils,
    Html,
    Accordion
) {
    'use strict';
    var addedCss = false;
    var DebugOverlay = function (settings) {
        var onReady = settings.onReady;
        var params = settings.params;
        var tests = [];
        var modules = {};
        var self = this;
        var onGetAllModules = function () {
            if (settings.showOverlay) {
                self.runTests();
                self.show();
            }

            if (onReady) {
                onReady();
            }
        };

        // collect tests
        Utils.forEach(params, function (param, path) {
            tests.push(path);
        });

        // acquire modules
        Utils.forEach(tests, function (path) {
            bento.require([path], function (Test) {
                var accordion = new Accordion({
                    title: path.replace('debug/tests/', ''),
                    isToggled: true
                });
                modules[path] = {
                    constructor: Test,
                    html: accordion
                };
                if (Utils.getKeyLength(modules) === tests.length) {
                    onGetAllModules();
                }
            }, function (err) {
                // failed to acquire module
                var accordion = new Accordion({
                    title: 'ERROR: could not load' + path,
                    isToggled: true
                });
                modules[path] = {
                    constructor: null,
                    html: accordion
                };
            });
        });

        this.modules = modules; // object
        this.tests = tests; // array
        this.params = params; // array
    };
    DebugOverlay.prototype.runTests = function () {
        var modules = this.modules;
        var params = this.params;
        // loop through test
        Utils.forEach(this.tests, function (path) {
            var Test = modules[path].constructor;
            var container = modules[path].html;
            var testFailed = false;
            var list = Html('ul', {
                style: {
                    'list-style-type': 'disc'
                }
            });
            var spawnTest = function () {
                // construct and run
                var tests = new Test(params[path] || {});
                Utils.forEach(tests, function (testObject) {
                    // name of unit test
                    var columnLeft = Html('p', {
                        display: 'inline',
                        innerHtml: testObject.name,
                        style: {
                            'margin-right': '2px'
                        }
                    });
                    // result of unit test
                    var columnRight = Html('p', {
                        display: 'inline',
                        // innerHtml: '<i class="fas fa-circle-notch fa-spin"></i>',
                        style: {
                            'margin-left': '8px'
                        }
                    });
                    var additionalDiv = Html('div');
                    var listItem = Html('li').attach(columnLeft, columnRight, additionalDiv);

                    list.attach(listItem);

                    // run the test and show result
                    testObject.run(additionalDiv, function (message) {
                        columnRight.innerHtml = message;
                        columnRight.style.color = 'green';
                    }, function (message) {
                        columnRight.innerHtml = message;
                        columnRight.style.color = 'red';
                        // if any test fails, add warning
                        if (!testFailed) {
                            testFailed = true;
                            container.title.innerHtml += ' <i class="fas fa-exclamation-triangle"></i>';
                        }
                    });
                });
            };

            container.attach(list);

            if (Test) {
                try {
                    spawnTest();
                } catch (e) {
                    container.attach(Html('<p>' + e + '</p>'), {
                        style: {
                            color: 'red'
                        }
                    });                    
                }
            } else {
                // show unavailable
                container.attach(Html('<p>Tests could not be initialized</p>'));
            }
        });
    };
    DebugOverlay.prototype.show = function () {
        // var nav = Html('nav', {
        //     id: 'navbar',
        //     class: 'navbar',
        //     role: 'navigation',
        //     'aria-label': 'main navigation',
        //     children: [
        //         Html('div', {
        //             class: 'navbar-item'
        //         })
        //     ]
        // });
        var modules = this.modules;
        var nav = Html('<nav class="navbar" role="navigation" aria-label="main navigation" />').attach(
            Html('<div class="navbar-brand" />').attach([
                Html('<a id="close" class="navbar-item" href="#"><i class="fas fa-times-circle"></i></a>', {
                    onclick: function () {
                        container.removeSelf();
                    }
                }),
                Html('<h1 id="frontpage-h1" class="title navbar-item">Debugging</h1>')
            ])
        );
        var container = Html('div', {
            id: 'debugContainer',
            style: {
                display: 'block',
                position: 'absolute',
                top: '0px',
                left: '0px',
                width: '100%',
                height: '100%',
                '-webkit-overflow-scrolling': 'touch',
                overflow: 'auto',
                'min-width': '100%',
                'min-height': '100%',
                'background-color': 'rgba(255, 255, 255, 0.9)'
            },
            children: [
                nav
            ]
        });
        Html.appendToBody(container);

        // load bulma.io style
        if (!addedCss) {
            document.head.insertAdjacentHTML('beforeend', '<link rel=stylesheet href=https://cdnjs.cloudflare.com/ajax/libs/bulma/0.7.5/css/bulma.min.css>');
            document.head.insertAdjacentHTML('beforeend', '<link rel="stylesheet" type="text/css" href="https://use.fontawesome.com/releases/v5.8.1/css/all.css" integrity="sha384-50oBUHEmvpQ+1lW4y57PTFmhCaXp0ML5d60M1M7uH2+nqUivzIebhndOJK28anvf" crossorigin="anonymous">');
            addedCss = true;
        }

        Utils.forEach(modules, function (module) {
            container.attach(module.html.container);
        });
    };

    return DebugOverlay;
});