/**
 * Module description
 * @moduleName Coin
 * @snippet Coin.snippet
Coin({
    position: new Vector2(0, 0)
})
 */
bento.define('entities/coin', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/culler',
    'components/spritecontainer',
    'entities/particle',
    'components/movable',
    'modules/savemanager',
    'components/gamecounter',
    'entities/effects/sparkleparticle'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    Culler,
    SpriteContainer,
    Particle,
    Movable,
    SaveManager,
    GameCounter,
    Sparkle
) {
    'use strict';
    return function (settings) {
        // --- PARAMETERS ---
        var identifier = settings.identifier;
        var startPosition = settings.position;
        var moveX = settings.moveX || 0;
        var moveY = settings.moveY || 0;
        var moveTimeX = settings.moveTime || settings.moveTimeX || 120;
        var moveTimeY = settings.moveTimeY || 120;
        var moveOffsetTimeX = settings.moveOffsetTime || settings.moveOffsetTimeX || 0;
        var moveOffsetTimeY = settings.moveOffsetTimeY || 0;
        var fall = false;
        var isCollected = !Globals.keepCoinsActive && (SaveManager.load('collectedCoinIdentifiers').indexOf(identifier) !== -1);
        var wasCollected = false;

        // --- FUNCTIONS ---
        var collect = function () {
            if (!isCollected) {
                var gameController = Bento.objects.get('gameController');
                if (gameController) {
                    gameController.giveCoins(1);
                    if (identifier) {
                        gameController.markCoinCollected(identifier);
                    }
                }
            }
            wasCollected = true;
            entity.removeSelf();
        };

        // --- COMPONENTS ---
        var movable = new Movable({
            velocity: new Vector2(0, 0),
            enabled: false,
            canCollideWith: ['solids'],
            stopOnCollide: false,
            onCollision: function (collisionData) {
                movable.velocity.y *= -0.5;
                if (Math.abs(movable.velocity.y) < 0.05) {
                    movable.velocity.y = 0;
                }
            }
        });

        // --- ENTITY ---
        var entity = new Entity({
            z: Globals.layers.active,
            name: 'coin',
            family: ['coins'],
            position: startPosition.clone(),
            boundingBox: new Rectangle(-8, -8, 16, 16),
            components: [
                new SpriteContainer({
                    spriteSheet: 'coin',
                    scale: Globals.pixelScaleV,
                    alpha: isCollected ? 0.35 : 1
                }),
                new Culler({
                    height: 8
                }),
                movable, {
                    name: "effectBehaviour",
                    update: function () {
                        if (moveX > 0) {
                            entity.position.x = startPosition.x + Math.sin(((entity.timer + moveOffsetTimeX) / moveTimeX) * Math.PI) * moveX;
                        }
                        if (moveY > 0) {
                            entity.position.y = startPosition.y + Math.sin(((entity.timer + moveOffsetTimeY) / moveTimeY) * Math.PI) * moveY;
                        }
                        if (fall) {
                            movable.velocity.y += 0.1;
                        }
                    },
                    destroy: function (data) {
                        if (!Globals.showEffects || !wasCollected) {
                            return;
                        }
                        new Particle({
                            z: Globals.layers.effects,
                            spriteSheet: 'effects/coin',
                            position: entity.position.clone(),
                            alpha: 1,
                            scale: Globals.pixelScaleV.scalarMultiply(1.5),
                            rotation: 0,
                            velocity: new Vector2(0, 0),
                            gravity: 0.05
                        });
                        new Tween({ in: 8,
                            onComplete: function () {
                                new Particle({
                                    z: Globals.layers.tiles - 1,
                                    imageName: 'ui/fx/bloom-halo',
                                    position: entity.position.clone(),
                                    scale: Globals.pixelScaleV,
                                    removeAfterTime: 10,
                                    originRelative: new Vector2(0.5, 0.5),
                                    removeEffect: 'scalefade'
                                });
                            }
                        });
                        //make sparkles
                        for (var i = 5 - 1; i >= 0; i--) {
                            new Sparkle({
                                position: entity.position.add(new Vector2(Utils.getRandomRangeFloat(-4, 4), Utils.getRandomRangeFloat(-4, 4)))
                            });
                        }
                        // make text particle
                        if (Globals.showCounters && !isCollected) {
                            new Particle({
                                z: Globals.layers.effects,
                                sprite: new GameCounter({
                                    value: '+1',
                                    imageName: 'addcoincounter'
                                }),
                                originRelative: new Vector2(0.5, 0.5),
                                position: entity.position.clone(),
                                rotation: 0,
                                rotationRate: Utils.getRandomRangeFloat(-0.02, 0.02),
                                velocity: new Vector2(Utils.getRandomRangeFloat(-0.5, 0.5), Math.min(movable.velocity.y, 0) - 1.5),
                                scale: Globals.pixelScaleV.scalarMultiply(0.45),
                                acceleration: new Vector2(0, 0.04),
                                friction: 1,
                                removeAfterTime: 60,
                                removeEffect: 'flicker'
                            });
                        }
                    },

                    draw: function (data) {
                        if (Globals.debug) {
                            var bb = entity.boundingBox;
                            data.renderer.fillRect([1, 0.75, 0, 0.5], bb.x, bb.y, bb.width, bb.height);
                        }
                    }
                }
            ]
        }).extend({
            collect: collect,
            startFall: function () {
                fall = true;
                movable.setEnabled(true);
                moveX = 0;
                moveY = 0;
            },
            setMoveX: function (newX) {
                moveX = newX;
            },
            setMoveY: function (newY) {
                moveY = newY;
            }
        });
        return entity;
    };
});