/**
 * Enemy that walks along the top of solids, turning at the corners
 * @moduleName SpiderEnemy
 * @snippet SpiderEnemy.snippet
SpiderEnemy({
    position: new Vector2(0, 0)
})
 */
bento.define('entities/spiderenemy', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/movable',
    'components/spritecontainer',
    'entities/particle'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    Movable,
    SpriteContainer,
    Particle
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();

        // --- VARS ---
        var camera;
        var position = settings.position;
        var spawnChance = settings.spawnChance || 1;
        var startX = position.x;
        var targetX = 90 + (settings.targetX || 0);

        var isDead = false;
        var isEnabled = true;
        var isGrounded = false;
        var wasGrounded = false;
        var gravity = 0.075;
        var jumpYSpeed = settings.jumpYSpeed || 2.5;
        var jumpXTween;
        var flipTween;


        // --- FUNCTIONS ---
        var setEnabled = function (newEnabled) {
            if (isEnabled !== newEnabled) {
                isEnabled = newEnabled;
            }
        };

        var squash = function () {
            if (isDead) {
                return;
            }
            if (jumpXTween) {
                jumpXTween.stop();
            }
            if (flipTween) {
                flipTween.stop();
            }
            isDead = true;
            setEnabled(false);
            movable.velocity.x = 0;
            movable.velocity.y = 2;
            movable.ignore.push('players');
            spriteContainer.sprite.setAnimation("idle");
            spriteContainer.sprite.setCurrentSpeed(0);
            Bento.audio.setVolume(0.25, 'sfx_squish');
            Bento.audio.playSound('sfx_hit');
            Bento.audio.playSound('sfx_squish');
            new Tween({
                from: new Vector2(Utils.sign(spriteContainerContainerContainer.scale.x) * 2, 0.35),
                to: new Vector2(Utils.sign(spriteContainerContainerContainer.scale.x) * 1.3, 0.8),
                in: 60,
                oscilations: 10,
                ease: 'easeOutElastic',
                onUpdate: function (v, t) {
                    spriteContainerContainerContainer.scale = v;
                }
            });
            new Tween({ in: 25,
                onComplete: function () {
                    for (var i = 4 - 1; i >= 0; i--) {
                        new Particle({
                            spriteSheet: 'particles/dust',
                            position: entity.position.add(new Vector2(Utils.getRandomRangeFloat(-8, 8), Utils.getRandomRangeFloat(-4, 4) - 2)),
                            alpha: 1,
                            scale: Globals.pixelScaleV,
                            rotation: Utils.getRandomRangeFloat(-2, 2),
                            rotationRate: 0,
                            velocity: new Vector2([-1, -1, 1, 1][i], Utils.getRandomRangeFloat(-0.1, 0.1)),
                            friction: 0.96,
                            removeAfterTime: 0,
                            removeEffect: 'none',
                            z: Globals.layers.effects
                        });
                    }
                    entity.removeSelf();
                }
            });
        };

        var chargeUp = function () {
            spriteContainer.sprite.setAnimation('chargeUp', jump);
        };

        var jump = function () {
            if (!isGrounded) {
                return;
            }

            startX = entity.position.x;
            var endX = Math.round(targetX - (entity.position.x - targetX));
            // jump up
            movable.velocity.y = -jumpYSpeed;
            // jump across
            if (jumpXTween) {
                jumpXTween.stop();
            }
            jumpXTween = new Tween({
                from: startX,
                to: endX,
                in: (2 * jumpYSpeed) / gravity,
                ease: 'linear',
                onUpdate: function (v, t) {
                    movable.velocity.x = (v - entity.position.x);
                },
                onComplete: function () {
                    movable.velocity.x = 0;
                    entity.position.x = endX;
                }
            });
            //swap end points
            startX = endX;

            //chage sprite
            spriteContainer.sprite.setAnimation('jumpUp');
            flipTween = new Tween({
                from: 0,
                to: Math.PI * 6 * -Utils.sign(entity.position.x - targetX),
                in: (2 * jumpYSpeed) / gravity,
                ease: 'easeOutQuad',
                onUpdate: function (v, t) {
                    spriteContainerContainer.rotation = v;
                }
            });
        };

        // --- COMPONENTS ---
        var spriteContainer = new SpriteContainer({
            spriteSheet: 'bugs/spider',
            position: new Vector2(0, 12),
            scale: new Vector2(Globals.pixelScale, Globals.pixelScale)
        });
        var spriteContainerContainer = new Entity({
            name: 'spriteContainerContainer',
            position: new Vector2(0, -12),
            components: [
                spriteContainer
            ]
        });
        var spriteContainerContainerContainer = new Entity({
            name: 'spriteContainerContainerContainer',
            components: [
                spriteContainerContainer
            ]
        });

        spriteContainer.sprite.setAnimation('idle');
        var behavior = {
            name: 'behaviorComponent',
            update: function (data) {
                if (isEnabled) {
                    // get stuff
                    var solids = [];
                    Utils.forEach(Bento.objects.getByFamily('solids'), function (solid, i, l, breakLoop) {
                        if (solid && solid.name !== 'player') {
                            solids.push(solid);
                        }
                    });
                    isGrounded = entity.collidesWith({
                        entities: solids,
                        offset: new Vector2(0, 1)
                    });

                    //land effect
                    if (!wasGrounded && isGrounded) {
                        if (movable.lastVelocity.y > 0) {
                            new Tween({
                                from: new Vector2(1.66, 0.33),
                                to: new Vector2(1, 1),
                                in: 30,
                                ease: 'easeOutBack',
                                onUpdate: function (v, t) {
                                    spriteContainer.scale = new Vector2(Globals.pixelScale, Globals.pixelScale).multiply(v);
                                }
                            });
                            if (Globals.showEffects) {
                                for (var i = 2 - 1; i >= 0; i--) {
                                    new Particle({
                                        spriteSheet: 'particles/dust',
                                        position: entity.position.add(new Vector2(Utils.getRandomRangeFloat(-4, 4), Utils.getRandomRangeFloat(-4, 4))),
                                        alpha: 1,
                                        scale: Globals.pixelScaleV,
                                        rotation: Utils.getRandomRangeFloat(-2, 2),
                                        rotationRate: 0,
                                        velocity: new Vector2(Utils.getRandomRangeFloat(-0.1, 0.1), Utils.getRandomRangeFloat(-0.1, 0.1)),
                                        gravity: 0,
                                        friction: 1,
                                        removeAfterTime: 0,
                                        removeEffect: 'none',
                                        z: Globals.layers.effects
                                    });
                                }
                            }
                        }
                        spriteContainer.sprite.setAnimation("idle");
                        // stop when landing
                        movable.velocity.x = 0;
                    }

                    var faceDir = -Utils.sign(entity.position.x - targetX);
                    if (isGrounded && faceDir !== 0) {
                        spriteContainerContainer.scale.x = faceDir;
                    }

                    //limit speed
                    movable.velocity.y += gravity * data.speed;
                    if (movable.velocity.y > jumpYSpeed) {
                        movable.velocity.y = jumpYSpeed;
                    }

                    if (spriteContainer.sprite.currentAnimation.name === "jumpUp" && movable.velocity.y >= 0) {
                        spriteContainer.sprite.setAnimation("jumpDown");
                    }

                    // jump
                    if (entity.ticker % 180 === 0) {
                        chargeUp();
                    }
                    wasGrounded = isGrounded;
                }
            }
        };

        // the movable the player requires to be able to move and hit solids
        var movable = new Movable({
            ignore: ['players', 'reviveclouds'],
            velocity: new Vector2(0, 0),
            stopOnCollide: true,
            checkInvisible: true,
            onCollision: function (cD) {
                if (cD.wasX) {
                    if (jumpXTween) {
                        jumpXTween.stop();
                    }
                    movable.velocity.x = -movable.lastVelocity.x * 0.66;
                }
            }
        });

        // --- ENTITY ---
        var entity = new Entity({
            z: Globals.layers.active,
            name: 'walkenemy',
            family: ['walkenemies', 'enemies'],
            position: position,
            boundingBox: new Rectangle(-12, -12, 24, 12),
            components: [
                behavior,
                movable,
                spriteContainerContainerContainer
            ]
        }).extend({
            spawnChance: spawnChance,
            squash: squash,
            getIsDead: function () {
                return isDead;
            }
        });
        return entity;
    };
});