/**
 * Module for atomic-plugins-googleplaygames
 * Tip: use https://github.com/LuckyKat/atomic-plugins-googleplaygames.git#update-gpg-lib:src/cordova
 * @moduleName GooglePlayGames
 */
bento.define('modules/googleplaygames', [
    'bento'
], function (
    Bento
) {
    'use strict';
    var gp;
    var clientId = 'xxxx.apps.googleusercontent.com';
    var defaultLeaderboard;
    var initialized = false;
    var isInitializing = false;
    var scopes = [
        'drive.appfolder'
    ];
    var nothing = function () {};
    var init = function (settings, onComplete) {
        if (isInitializing) {
            // already trying to initialize
            return;
        }
        clientId = settings.clientId;
        if (!clientId) {
            console.error('Error: enter the OAuth2 clientId. You can find this in the GPG dashboard under Linked Apps.');
        }
        defaultLeaderboard = settings.defaultLeaderboard;
        scopes = settings.scopes || [];
        isInitializing = true;
        // set default player stats if none are available (eg. on first bootup or iOS)
        if (!Bento.saveState.load('playerStats', null)) {
            Bento.saveState.save('playerStats', {
                averageSessionLength: -1, // -1 is the default value when there's not enough data
                churnProbability: -1,
                daysSinceLastPlayed: -1,
                highSpenderProbability: -1,
                numberOfPurchases: -1,
                numberOfSessions: -1,
                sessionPercentile: -1,
                spendPercentile: -1,
                spendProbability: -1,
                totalSpendNext28Days: -1
            });
        }

        gp = window.Cocoon.Social.GooglePlayGames;
        gp.init({
            clientId: clientId,
            defaultLeaderboard: defaultLeaderboard,
            showAchievementNotifications: true,
            /**
             * WARNING: Be sure to also enable the scopes on console.developers.google.com,
             * otherwise you will get an 'INVALID_SCOPE' login error.
             */
            scopes: scopes
        }, function (error) {
            isInitializing = false;
            initialized = true;
            if (onComplete) {
                onComplete(error);
            }
        });
    };
    var isLoggedIn = function () {
        if (!initialized) {
            return false;
        }
        var socialService = gp.getSocialInterface();
        console.log('Google Play: is logged in', socialService.isLoggedIn());
        return socialService.isLoggedIn();
    };
    var login = function (onComplete) {
        var socialService;
        onComplete = onComplete || function () {};
        socialService = gp.getSocialInterface();
        socialService.login(onComplete);
    };
    var dummy = {
        init: nothing,
        isLoggedIn: nothing,
        login: nothing,
        post: nothing,
        show: nothing,
        postAchievement: nothing,
        showAchievement: nothing,
        submitEvent: nothing,
        loadSavedGame: nothing,
        writeSavedGame: nothing,
        showSavedGame: nothing,
        loadPlayerStats: nothing
    };
    var GooglePlayGames = {
        /**
         * @snippet GooglePlayGames.init()|snippet
GooglePlayGames.init({
    clientId: 'xxxx.apps.googleusercontent.com',
    defaultLeaderboard: '', // delete if not used
    scopes: ['drive.appfolder'] // delete if not using GPG Saved Games
}, function () {});
         */
        init: function (settings, onComplete) {
            if (!initialized || isInitializing) {
                init(settings, onComplete);
            }
        },
        /**
         * @snippet GooglePlayGames.isLoggedIn()|Boolean
GooglePlayGames.isLoggedIn();
         */
        isLoggedIn: isLoggedIn,
        /**
         * @snippet GooglePlayGames.login()|snippet
GooglePlayGames.login(function (success, error) {
    if (!success) {
        console.log(error);
        return;
    }
    $1
});
         */
        login: function (onComplete) {
            var socialService,
                self = this;

            if (!initialized) {
                init(function (error) {
                    self.login(onComplete);
                });
                return;
            }

            socialService = gp.getSocialInterface();
            if (socialService.isLoggedIn()) {
                // dont log in twice
                if (onComplete) {
                    onComplete(false);
                }
                return;
            }
            socialService.login(function (loggedIn, error) {
                if (error) {
                    if (onComplete) {
                        onComplete(false, error);
                    }
                    console.error("login error: " + error.message + " " + error.code);
                } else {
                    if (onComplete) {
                        onComplete(loggedIn);
                    }
                }
            });
        },
        /**
         * @snippet GooglePlayGames.post()|snippet
GooglePlayGames.post(${1:0}, '${2:leaderboardId}');
         */
        post: function (v, leaderboardId, onComplete) {
            if (!initialized) {
                return;
            }
            var socialService = gp.getSocialInterface();
            if (isLoggedIn()) {
                socialService.submitScore(v, onComplete, {
                    leaderboardID: leaderboardId
                });
            }
        },
        /**
         * @snippet GooglePlayGames.show()|snippet
GooglePlayGames.show('${1:leaderboardId}');
         */
        show: function (id, onComplete) {
            if (!initialized) {
                return;
            }
            var socialService = gp.getSocialInterface();

            if (!isLoggedIn()) {
                login(function () {
                    console.log('logged in');
                    socialService.showLeaderboard(function (err) {
                        console.log('show leaderboard', JSON.stringify(err));
                    }, {
                        leaderboardID: id
                    });
                });
            } else {
                socialService.showLeaderboard(function (err) {
                    console.log('show leaderboard', JSON.stringify(err));
                }, {
                    leaderboardID: id
                });
            }

        },
        /**
         * @snippet GooglePlayGames.postAchievement()|snippet
        GooglePlayGames.postAchievement('${1:achievementId}');
         */
        postAchievement: function (id, onComplete) {
            if (!initialized) {
                return;
            }
            var socialService = gp.getSocialInterface();
            if (isLoggedIn()) {
                socialService.submitAchievement(id, function (response) {
                    console.log('achievement unlocked', response);
                    if (response && response.newlyUnlocked && onComplete) {
                        // show message
                        onComplete();
                    }
                });
            }
        },
        /**
         * @snippet GooglePlayGames.showAchievement()|snippet
        GooglePlayGames.showAchievement(function () {});
         */
        showAchievement: function (onComplete) {
            if (!initialized) {
                return;
            }
            var socialService = gp.getSocialInterface();
            if (!isLoggedIn()) {
                login(function () {
                    socialService.showAchievements(function (err) {
                        console.log('show leaderboard', JSON.stringify(err));
                    });
                });
            } else {
                socialService.showAchievements(function (err) {
                    console.log('show leaderboard', JSON.stringify(err));
                });

            }
        },
        /**
         * @snippet GooglePlayGames.submitEvent()|snippet
        GooglePlayGames.submitEvent('${1:eventId}', ${2:1});
         */
        submitEvent: function (name, amount) {
            if (!initialized) {
                return;
            }
            if (window.Cocoon.Social.GooglePlayGames.submitEvent) {
                window.Cocoon.Social.GooglePlayGames.submitEvent(name, amount);
            } else {
                console.log('submitEvent not available (cocoon devapp?)');
            }
        },
        /**
         * @snippet GooglePlayGames.loadSavedGame()|snippet
GooglePlayGames.loadSavedGame('${1:id}', function (snapshot) {
    // success
}, function (error) {
    // fail
});
        */
        loadSavedGame: function (name, onSuccess, onFail) {
            if (!initialized) {
                return;
            }
            window.Cocoon.Social.GooglePlayGames.loadSavedGame(name, function (snapshot, error) {
                if (error && onFail) {
                    onFail(error); // error should be a json object
                    return;
                }
                if (onSuccess) {
                    // console.log('load snapshot', JSON.stringify(snapshot));
                    if (snapshot.data) {
                        onSuccess(snapshot);
                    } else {
                        onFail(snapshot);
                    }
                }
            });
        },
        /**
         * @snippet GooglePlayGames.writeSavedGame()|snippet
GooglePlayGames.writeSavedGame(${1:snapshotObject}, function () {
    // success
}, function (error) {
    // fail
});
        */
        writeSavedGame: function (snapshot, onSuccess, onFail) {
            if (!initialized) {
                return;
            }
            /*snapshot {
                identifier: string,
                title: string,
                description: string,
                data: string //this is the JSON you want to load/store
            }*/
            window.Cocoon.Social.GooglePlayGames.writeSavedGame(snapshot, function (error) {
                if (error && onFail) {
                    onFail(error); // error should be a json object
                    return;
                }
                if (onSuccess) {
                    onSuccess();
                }
            });
        },
        /**
         * @snippet GooglePlayGames.showSavedGame()|snippet
GooglePlayGames.showSavedGame(${1:snapshotObject});
        */
        showSavedGame: function (snapshot) {
            if (!initialized) {
                return;
            }
            window.Cocoon.Social.GooglePlayGames.showSavedGames(snapshot, function (error) {

            });
        },
        /**
         * @snippet GooglePlayGames.loadPlayerStats()|snippet
GooglePlayGames.loadPlayerStats(function (playerStats) {});
        */
        loadPlayerStats: function (onComplete) {
            if (!initialized) {
                return;
            }
            window.Cocoon.Social.GooglePlayGames.loadPlayerStats(function (stats, error) {
                if (stats) {
                    if (onComplete) {
                        onComplete(stats);
                    }
                }
                if (error) {
                    console.error(error);

                    if (onComplete) {
                        onComplete(null);
                    }
                }
            });
        },
        /**
         * Loads the scores around a player for the specified leaderboard id.
         *
         * @param {String} leaderboardID - The leaderboard identifier to get score from.
         * @param {Boolean} topScores - If true, will load top scores. If false, will load scores centered around user. Defaults to true.
         * @param {Number} timeSpan - For which time period to get scores. Use 0 for daily scores, 1 for weekly scores, 2 for scores since the start of the leaderboard. Defaults to 2.
         * @param {Boolean} friends - If true, will load scores from user's social circles. If false, will load general scores. Defaults to false.
         * @param {Function} onComplete - The callback function. It receives the following parameters:
         * @param {Object[]} scores - Array of objects containing leaderboard info about players. Example:
         *                            [{
         *                                "id": 123456789098765432101,
         *                                "name": "Johnny B. Goode",
         *                                "rank": 1,
         *                                "score": 1234
         *                            }, {
         *                                ...
         *                            },
         *                            ...
         *                            ]
         * @snippet GooglePlayGames.loadScores()|snippet
GooglePlayGames.loadScores({
    leaderboardId: '',
    topScores: true, // loads top scores. If false, will load scores centered around user.
    timeSpan: 2, // Use 0 for daily scores, 1 for weekly scores, 2 for scores since the start of the leaderboard
    friends: false // If true, will load scores from user's social circles.
});
         */
        loadScores: function (settings, onComplete) {
            if (!initialized) {
                return;
            }

            var topScores = settings.topScores === false ? false : true;
            var timeSpan = settings.timeSpan === 0 ? 0 : (settings.timeSpan || 2);
            var friends = settings.friends ? true : false;

            var cordovaCallback = function (scores, error) {
                if (scores) {

                    // TODO: save the leaderboard using Bento.saveState?

                    if (onComplete) {
                        onComplete(scores);
                    }
                }
                if (error) {
                    if (onComplete) {
                        onComplete(null);
                    }
                }
            };

            if (topScores) {
                window.Cocoon.Social.GooglePlayGames.loadTopScores(settings.leaderboardId, timeSpan, friends, cordovaCallback);
            } else {
                window.Cocoon.Social.GooglePlayGames.loadPlayerCenteredScores(settings.leaderboardId, timeSpan, friends, cordovaCallback);
            }
        }
    };

    if (!window.Cocoon || !window.Cocoon.Social || !window.Cocoon.Social.GooglePlayGames) {
        return dummy;
    }
    return GooglePlayGames;
});