/**
 * Module description
 * @moduleName GameGUI
 * @snippet GameGUI.snippet
GameGUI({
    position: new Vector2(0, 0)
})
 */
bento.define('ui/gamegui', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'components/spritecontainer',
    'components/gamecounter',
    'ui/cointracker',
    'entities/particle',
    'modules/savemanager',
    'modules/skinmanager',
    'modules/levelmanager'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    SpriteContainer,
    GameCounter,
    CoinTracker,
    Particle,
    SaveManager,
    SkinManager,
    LevelManager
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var gameController = Bento.objects.get('gameController');
        var viewMid = new Vector2(viewport.width * 0.5, viewport.height * 0.5);

        // --- PARAMETERS ---
        var position = new Vector2(0, 0);

        // --- VARS ---
        var levelProgress = 0;
        var currentLevel = LevelManager.getCurrentLevel();
        var currentScore = 0;
        var superProgress = SaveManager.load('superProgress');
        var potSkinsThisLevel = SkinManager.getPotSkinsForLevel(currentLevel);
        var plantSkins = SkinManager.getAllLockedPlantSkins(true);
        var isSpecialStage = gameController.getIsSpecialStage();

        var isHidden = false;
        var hide = function (snap) {
            if (isHidden) {
                return;
            }
            isHidden = true;
            if (snap) {
                entity.position = position.add(new Vector2(0, -128));
                return;
            }
            new Tween({
                from: entity.position.clone(),
                to: position.add(new Vector2(0, -128)),
                in: 20,
                ease: 'easeInBack',
                onUpdate: function (v, t) {
                    entity.position = v;
                }
            });
        };
        var show = function (snap) {
            if (!isHidden) {
                return;
            }
            isHidden = false;
            if (snap) {
                entity.position = position;
                return;
            }
            new Tween({
                from: entity.position.clone(),
                to: position.clone(),
                in: 20,
                ease: 'easeOutBack',
                onUpdate: function (v, t) {
                    entity.position = v;
                }
            });
        };
        // --- COMPONENTS ---
        var statusBG = new SpriteContainer({
            imageName: 'ui/status-back',
            originRelative: new Vector2(0.5, 0),
            position: new Vector2(viewMid.x, isSpecialStage ? -20 : -6),
            scale: new Vector2(Globals.pixelScaleUI * 2, Globals.pixelScaleUI * 2)
        });

        var levelCurrentText = new Text({
            fontSettings: Utils.getTextStyle('levelText'),
            position: new Vector2(viewMid.x - 52.5, 43),
            text: currentLevel.toString(),
            maxWidth: 16,
            maxHeight: 16
        });

        var levelNextText = new Text({
            fontSettings: Utils.getTextStyle('levelText'),
            position: new Vector2(viewMid.x + 52.5, 43),
            text: (currentLevel + 1).toString(),
            maxWidth: 16,
            maxHeight: 16
        });

        var potPresentSprite = new SpriteContainer({
            spriteSheet: 'boxes/' + (potSkinsThisLevel[0] || 'default'),
            position: new Vector2(viewMid.x + 52, 41),
            scale: Globals.pixelScaleUIV
        }).attach({
            name: 'wobbleBehaviour',
            update: function () {
                this.parent.rotation = Math.sin(this.parent.timer * 0.1) * 0.2;
            }
        });

        var scoreBacking = new SpriteContainer({
            imageName: 'ui/scoreholder',
            originRelative: new Vector2(0.5, 0.5),
            position: new Vector2(viewMid.x, 48),
            alpha: 1,
            scale: new Vector2(Globals.pixelScaleUI, Globals.pixelScaleUI),
            rotation: 0
        });
        var scoreCounter = new GameCounter({
            position: new Vector2(0, 176),
            align: 'center',
            value: currentScore,
            scale: new Vector2(0.8, 0.8)
        });
        scoreCounter.setValue(0);
        scoreBacking.attach(scoreCounter);

        var superProgressBack = new SpriteContainer({
            name: 'superProgressBack',
            imageName: 'ui/beanprogress',
            position: new Vector2(0, 12),
            originRelative: new Vector2(0.5, 0.5),
        });
        var superProgressFilled = Bento.assets.getImage('ui/beanprogress-filled');
        var totalWidth = superProgressFilled.width;
        var superMarker = new SpriteContainer({
            name: 'superMarker',
            imageName: 'ui/icons/super-marker',
            position: new Vector2(0, -40),
            originRelative: new Vector2(0.5, 0.5),
            segment: -1
        });
        var super2Icon = new SpriteContainer({
            name: 'super2Icon',
            spriteSheet: 'beans/' + (plantSkins[0] || 'daisy'),
            position: new Vector2(210, -30),
            scale: new Vector2(0.66, 0.66)
        });
        var super1Icon = new SpriteContainer({
            name: 'super1Icon',
            spriteSheet: 'ui/small-chest',
            position: new Vector2(0, -30),
            scale: new Vector2(0.66, 0.66)
        });
        super1Icon.sprite.setAnimation('closed');
        if (superProgress > Math.round(Globals.superProgressSegments * 0.5) - 1) {
            super1Icon.alpha = 0;
        }
        if (SaveManager.load('hasInitialBean') && SaveManager.load('supersComplete') === 0) {
            super1Icon.alpha = 0;
        }
        var superProgressUI = new Entity({
            name: 'superProgressUI',
            position: new Vector2(viewMid.x, 36),
            scale: Globals.pixelScaleUIV.clone(),
            components: [
                superProgressBack, {
                    name: 'superProgressBehaviour',
                    update: function (data) {
                        superMarker.position.y = -60 + (Math.sin(superMarker.ticker * 0.2) * 5);
                        if (superProgress !== this.segment) {
                            this.segment = superProgress;
                            var displayProgress = Utils.clamp(0, superProgress, Globals.superProgressSegments - 1);
                            var idealPosition = new Vector2(((displayProgress + 0.5) / Globals.superProgressSegments - 0.5) * totalWidth, (displayProgress === Globals.superProgressSegments - 1) ? -80 : -25);
                            new Tween({
                                from: superMarker.position.clone(),
                                to: idealPosition,
                                in: 15,
                                ease: 'easeInOutQuad',
                                onUpdate: function (v, t) {
                                    superMarker.position = v;
                                }
                            });
                        }
                        if (superProgress === Globals.superProgressChest - 1) {
                            super1Icon.scale.x = 0.66 + Math.sin(superMarker.timer * 0.2) * 0.1 * (levelProgress + 0.2);
                            super1Icon.scale.y = 0.66 + Math.cos(superMarker.timer * 0.2) * 0.1 * (levelProgress + 0.2);
                            super1Icon.rotation = Math.cos(superMarker.timer * 0.33) * 0.1 * (levelProgress + 0.2);
                        }
                        if (superProgress >= Globals.superProgressSegments - 1) {
                            super2Icon.scale.x = 0.66 + Math.sin(superMarker.timer * 0.2) * 0.1 * (levelProgress + 0.2);
                            super2Icon.scale.y = 0.66 + Math.cos(superMarker.timer * 0.2) * 0.1 * (levelProgress + 0.2);
                            super2Icon.rotation = Math.cos(superMarker.timer * 0.33) * 0.1 * (levelProgress + 0.2);
                        }
                    },
                    draw: function (data) {
                        var r = data.renderer;
                        var img = superProgressFilled;
                        var percentComplete = Math.min(1, (1 / Globals.superProgressSegments * superProgress + levelProgress / Globals.superProgressSegments));
                        r.drawImage(
                            img,
                            0,
                            0,
                            img.width * percentComplete,
                            img.height,
                            0 - (img.width * 0.5), -22,
                            img.width * percentComplete,
                            img.height * 1
                        );
                    },
                },
                super1Icon,
                super2Icon,
                superMarker
            ]
        });

        if (!SaveManager.load('hasInitialBean')) {
            super1Icon.sprite.setSpriteSheet('beans/' + plantSkins[0]);
            super2Icon.sprite.setSpriteSheet('beans/' + plantSkins[1]);
        }

        var statusBarBack = new SpriteContainer({
            name: 'statusBarBack',
            imageName: 'ui/statusbar-back',
            position: new Vector2(viewMid.x, 43),
            originRelative: new Vector2(0.5, 0.5),
            scale: Globals.pixelScaleUIV
        });
        var statusBarImage = Bento.assets.getImage('ui/statusbar-filled');
        var statusBarFrames = 9;
        var statusBarFrameNow = 0;
        var statusBarBehaviour = {
            name: 'statusBarBehaviour',
            position: new Vector2(viewMid.x, 46),
            scale: new Vector2(Globals.pixelScaleUI, Globals.pixelScaleUI),
            percent: 0,
            update: function (data) {
                statusBarFrameNow += 0.5 * data.speed;
                statusBarFrameNow = statusBarFrameNow % statusBarFrames;
                this.percent = levelProgress;
            },
            draw: function (data) {
                var r = data.renderer;
                var img = statusBarImage;
                r.drawImage(
                    img,
                    0,
                    0 + (Math.floor(statusBarFrameNow) * img.height * (1 / statusBarFrames)),
                    img.width * this.percent,
                    img.height * (1 / statusBarFrames),
                    this.position.x - (img.width * 0.5 * this.scale.x),
                    this.position.y - (img.height * (1 / statusBarFrames) * 0.5 * this.scale.y),
                    img.width * this.scale.x * this.percent,
                    img.height * this.scale.y * (1 / statusBarFrames)
                );
            }
        };
        // holder for the status bar
        var statusBar = new Entity({
            name: 'statusBar',
            position: new Vector2(0, 0),
            components: [
                statusBarBack,
                levelCurrentText,
                (potSkinsThisLevel.length === 0) ? levelNextText : potPresentSprite,
                statusBarBehaviour
            ]
        });

        var specialCoinBacking = new SpriteContainer({
            imageName: 'ui/scoreholder',
            originRelative: new Vector2(0.5, 0.5),
            position: new Vector2(viewMid.x, 48),
            scale: new Vector2(Globals.pixelScaleUI * 0.75, Globals.pixelScaleUI * 0.75)
        });
        var specialCoinCounter = new GameCounter({
            imageName: "addcoincounter",
            position: new Vector2(-4, 176),
            align: 'center',
            value: 0,
            scale: new Vector2(1, 1)
        });
        var specialCoinSprite = new SpriteContainer({
            imageName: 'ui/big_coin',
            position: new Vector2(0, -60),
            originRelative: new Vector2(0.5, 0.5),
            scale: new Vector2(0.4, 0.4)
        });
        var specialCoinBehaviour = {
            name: "specialCoinBehaviour",
            update: function () {
                if (specialCoinCounter.getValue() !== gameController.getCoinsCollected()) {
                    specialCoinCounter.setValue(gameController.getCoinsCollected());
                }
            }
        };
        specialCoinCounter.setValue(0);
        specialCoinBacking.attach(specialCoinCounter);
        specialCoinBacking.attach(specialCoinSprite);
        specialCoinBacking.attach(specialCoinBehaviour);


        var behavior = {
            name: 'behaviorComponent',
            update: function (data) {
                if (!gameController) {
                    gameController = Bento.objects.get('gameController');
                    if (!gameController) {
                        return;
                    }
                }
                levelProgress = gameController.getLevelPercent();
                currentScore = Globals.currentScore;
                if (scoreCounter.getValue() !== currentScore) {
                    var scoreDiff = currentScore - scoreCounter.getValue();
                    if (scoreDiff > 0) {
                        if (scoreDiff > 300) {
                            scoreCounter.setValue(scoreCounter.getValue() + 100);
                        }
                        if (scoreDiff > 30) {
                            scoreCounter.setValue(scoreCounter.getValue() + 10);
                        }
                        scoreCounter.setValue(scoreCounter.getValue() + 1);
                    } else {
                        scoreCounter.setValue(currentScore);
                    }
                }
                entity.alpha = Globals.showGUI ? 1 : 0;
            }
        };

        var coinTracker = new CoinTracker({
            position: new Vector2(viewport.width - 12, 19),
            playSound: true
        });

        var superChest = new SpriteContainer({
            spriteSheet: 'ui/small-chest',
            originRelative: new Vector2(0.5, 0.5),
            position: new Vector2(20, 24),
            scale: new Vector2(0.15, 0.15)
        });
        var superChestWobble = {
            name: "superChestWobble",
            update: function () {
                superChest.rotation = Math.sin(superChest.ticker * 0.1) * 0.1;
            }
        };

        var superChestAnimation = function () {
            entity.attach(superChest);
            entity.attach(superChestWobble);
            new Tween({
                from: 0,
                to: superChest.scale.x,
                in: 15,
                ease: 'easeOutBack',
                onUpdate: function (v, t) {
                    superChest.scale.x = v;
                    superChest.scale.y = v;
                }
            });
            superChest.scale = new Vector2(0, 0);
        };

        // --- ENTITY ---
        var entity = new Entity({
            z: Globals.layers.gui,
            name: 'gameGUI',
            position: position,
            float: true,
            components: [
                (!isSpecialStage) ? scoreBacking : {
                    name: 'blank'
                },
                statusBG,
                (!isSpecialStage) ? statusBar : specialCoinBacking,
                behavior,
                (plantSkins.length > 0 && !isSpecialStage) ? superProgressUI : {
                    name: 'blank'
                },
                (!isSpecialStage) ? coinTracker : {
                    name: 'blank'
                },
            ]
        }).extend({
            hide: hide,
            show: show,
            getCoinTracker: function () {
                return coinTracker;
            },
            showScore: function (dontAnimate) {
                if (dontAnimate) {
                    scoreBacking.position.y = 48;
                } else {
                    new Tween({
                        from: scoreBacking.position.y,
                        to: 48,
                        in: 15,
                        ease: 'easeOutQuad',
                        onUpdate: function (v, t) {
                            scoreBacking.position.y = v;
                        }
                    });
                }
            },
            hideScore: function (dontAnimate) {
                if (dontAnimate) {
                    scoreBacking.position.y = 0;
                } else {
                    new Tween({
                        from: scoreBacking.position.y,
                        to: 0,
                        in: 15,
                        ease: 'easeOutQuad',
                        onUpdate: function (v, t) {
                            scoreBacking.position.y = v;
                        }
                    });
                }
            },
            superChestAnimation: superChestAnimation,
            getScoreBacking: function () {
                return scoreBacking;
            },
            getScoreCounter: function () {
                return scoreCounter;
            },
            super1Icon: super1Icon,
            super2Icon: super2Icon
        });
        return entity;
    };
});