/**
 * Module description
 * @moduleName LevelSelector
 * @snippet LevelSelector.snippet
LevelSelector({
    position: new Vector2(0, 0)
})
 */
bento.define('ui/levelselector', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'components/spritecontainer',
    'globals',
    'ui/levelnode',
    'modules/levelmanager'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    SpriteContainer,
    Globals,
    LevelNode,
    LevelManager
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();

        // --- PARAMETERS ---
        var worldNumber = settings.worldNumber || 0;
        var addInfinite = settings.addInfinite || false;
        var isScrolling = settings.isScrolling || function () {
            return false;
        };
        var currentLevel = LevelManager.getCurrentLevel();
        var selectionIndicatorSpinRate = 0.025;

        // --- VARS ---
        // list of node positions on the map
        var levelNodePositions = [
            new Vector2(-39, 40),
            new Vector2(0, 40),
            new Vector2(39, 40),
            new Vector2(58.5, 0),
            new Vector2(19.5, 0),
            new Vector2(-19.5, 0),
            new Vector2(-58.5, 0),
            new Vector2(-39, -40),
            new Vector2(0, -40),
            new Vector2(39, -40)
        ];
        var infiniteNodePosition = new Vector2(39, -80);

        // list of all the nodes
        var levelNodes = [];

        // --- FUNCTIONS ---
        var getWorldPos = function (e) {
            if (!e) {
                return;
            }
            var pos = e.position.clone();
            var lastEnt = e;
            while (lastEnt.parent) {
                pos.addTo(lastEnt.parent.position);
                lastEnt = lastEnt.parent;
            }
            return pos;
        };

        // --- COMPONENTS ---
        var selectionIndicator = new SpriteContainer({
            imageName: 'ui/sagamap/selection',
            originRelative: new Vector2(0.5, 0.5),
            scale: Globals.pixelScaleUIV
        }).attach({
            name: 'selectionIndicatorBehaviour',
            update: function () {
                if (selectionIndicator.visible) {
                    selectionIndicator.rotation += selectionIndicatorSpinRate;
                }
            }
        });
        selectionIndicator.visible = false;

        // holding entity for the nodes
        var levelNodeEntity = new Entity({
            name: 'levelNodeEntity'
        });

        // recreates the map nodes 
        var generateLevelNodes = function () {
            //clear old map nodes
            Utils.forEach(levelNodes, function (levelNode, i, l, breakLoop) {
                levelNode.removeSelf();
            });

            // clear array of nodes
            levelNodes = [];

            //make all the new ones
            Utils.forEach(levelNodePositions, function (levelNodePosition, i, l, breakLoop) {
                //make the node
                var thisLevel = (worldNumber * 10) + i + 1;
                var thisLevelNode = new LevelNode({
                    levelNumber: thisLevel,
                    position: levelNodePosition.clone(),
                    onSelect: function () {
                        if (isScrolling()) {
                            return;
                        }
                        //change level
                        if (thisLevel !== currentLevel) {
                            LevelManager.changeLevel(thisLevel, getWorldPos(levelSelector).add(levelNodePosition));
                        }
                        selectLevelNode(thisLevel);
                    }
                });
                //attach the node
                levelNodeEntity.attach(thisLevelNode);

                // push to array of nodes
                levelNodes.push(thisLevelNode);
            });
            if (addInfinite) {
                //make the node
                var infiniteLevel = LevelManager.getInfiniteModeLevel();
                var infiniteNode = new LevelNode({
                    levelNumber: infiniteLevel,
                    isInfinite: true,
                    position: infiniteNodePosition.clone(),
                    onSelect: function () {
                        if (isScrolling()) {
                            return;
                        }
                        //change level
                        if (infiniteLevel !== currentLevel) {
                            LevelManager.changeLevel(infiniteLevel, getWorldPos(levelSelector).add(infiniteNodePosition));
                        }
                        selectLevelNode(infiniteLevel);
                    }
                });

                //attach the node
                levelNodeEntity.attach(infiniteNode);
                // push to array of nodes
                levelNodes.push(infiniteNode);
            }
        };

        var selectLevelNode = function (level) {
            Utils.forEach(levelNodes, function (levelNode, i, l, breakLoop) {
                if (levelNode.getLevelNumber() === level) {
                    //appear & move
                    selectionIndicator.visible = true;
                    selectionIndicator.position = levelNode.position.clone();

                    //scale effect
                    new Tween({
                        from: 2,
                        to: 1,
                        in: 20,
                        ease: 'easeOutBack',
                         
                        onUpdate: function (v, t) {
                            selectionIndicator.scale = Globals.pixelScaleUIV.scalarMultiply(v);
                        }
                    });
                    new Tween({
                        from: 0.1,
                        to: 0.025,
                        in: 60,
                        ease: 'easeOutQuad',
                        onUpdate: function (v, t) {
                            selectionIndicatorSpinRate = v;
                        }
                    });

                    breakLoop();
                }
            });
        };

        var levelSelector = new Entity({
            name: 'levelSelector',
            position: settings.position || new Vector2(0, 0),
            components: [
                new SpriteContainer({
                    imageName: 'ui/sagamap/pipemap' + (1 + worldNumber),
                    originRelative: new Vector2(0.5, 0.5),
                    position: new Vector2(0, 0),
                    alpha: 1,
                    scale: Globals.pixelScaleUIV.scalarMultiply(2.8),
                    rotation: 0
                }), {
                    name: 'visibiltyBehaviour',
                    update: function () {
                        var worldPos = getWorldPos(levelSelector);
                        levelSelector.visible = (worldPos.x - 75 < viewport.width) && (worldPos.x + 75 > viewport.x);
                    }
                },
                levelNodeEntity,
                selectionIndicator
            ]
        });
        generateLevelNodes();
        selectLevelNode(currentLevel);

        return levelSelector;
    };
});