/**
 * Module description
 * @moduleName SuperChestButton
 * @snippet SuperChestButton.snippet
SuperChestButton({
    position: new Vector2(0, 0)
})
 */
bento.define('ui/superchestbutton', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'globals',
    'modules/ads',
    'components/spritecontainer',
    'modules/savemanager',
    'ui/superchestscreen',
    'modules/skinmanager'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    Globals,
    Ads,
    SpriteContainer,
    SaveManager,
    SuperChestScreen,
    SkinManager
) {
    'use strict';
    return function (settings) {
        // --- VARS ---
        var buttonScalar = new Vector2(0.53 * 1.15, 1.15).scalarMultiply(0.9);
        var wasActive;
        var isWatchingAd = false;
        var canWiggle = true;

        // --- COMPONENTS ---
        var contents = new Entity({
            name: 'contents',
            position: new Vector2(0, 0),
            scale: new Vector2(1, 1).divide(buttonScalar),
            components: [
                new SpriteContainer({
                    spriteSheet: 'ui/small-chest',
                    scale: new Vector2(0.9, 0.9),
                    position: new Vector2(0, 24)
                }),
                new SpriteContainer({
                    imageName: 'ui/icons/vid',
                    originRelative: new Vector2(0.5, 0.5),
                    position: new Vector2(-64, -64),
                    scale: new Vector2(0.3, 0.3),
                    rotation: -0.5
                })
            ]
        });
        var buttonBehaviour = {
            name: 'buttonbehaviour',
            update: function () {
                var isActive = Ads.canShowRewarded() && !isWatchingAd && SaveManager.load('hasSuperChestReady');
                if (wasActive !== isActive) {
                    button.setActive(isActive);
                }
                wasActive = isActive;
            }
        };

        var motionBehaviour = {
            name: 'motionBehaviour',
            start: function () {
                var p = this.parent;
                var wiggle = function () {
                    new Tween({
                        from: 1,
                        to: 0,
                        in: 30,
                        applyOnDelay: 0,
                        ease: 'easeOutQuad',
                        onUpdate: function (v, t) {
                            if (canWiggle) {
                                p.scale = Globals.pixelScaleV.multiply(buttonScalar).scalarMultiply(1 + (v * 0.125));
                            }
                        }
                    });
                    new Tween({
                        from: 1,
                        to: 0,
                        in: 60,
                        applyOnDelay: 0,
                        ease: 'easeOutQuad',
                        onUpdate: function (v, t) {
                            p.rotation = Math.sin(p.timer * 0.5) * 0.1 * v;
                        },
                        onComplete: function () {
                            new Tween({ in: 15,
                                onComplete: function () {
                                    if (canWiggle) {
                                        wiggle();
                                    }
                                }
                            });
                        }
                    });
                };
                new Tween({ in: 65,
                    onComplete: function () {
                        wiggle();
                    }
                });

            }
        };

        var giveSuperChest = function () {
            canWiggle = false;
            new Tween({
                from: button.scale.clone(),
                to: new Vector2(0.001, 0.001),
                in: 15,
                ease: 'easeOutBack',
                onUpdate: function (v, t) {
                    button.scale = v;
                },
                onComplete: function () {
                    button.alpha = 0;
                }
            });
            new SuperChestScreen({
                skipAd: true,
                onComplete: function () {
                    var player = Bento.objects.get('player');
                    if (player) {
                        player.setPotSkin(SkinManager.getCurrentPotSkin());
                    }
                    var mainMenuScreen = Bento.objects.get('mainMenuScreen');
                    if (mainMenuScreen) {
                        mainMenuScreen.potSelector.generateSlots();
                    }
                }
            });
            SaveManager.save('hasSuperChestReady', false);
            SaveManager.save('superProgressNextChest', (SaveManager.load('superProgress') + 4) % Globals.superProgressSegments);
            if (SaveManager.load('superProgressNextChest') >= Globals.superProgressSegments - 1) {
                SaveManager.save('superProgressNextChest', Globals.superProgressSegments - 2);
            }
        };

        // --- ENTITY ---
        var button = new ClickButton({
            name: 'collectButton',
            sfx: 'sfx_clickbutton',
            imageName: 'ui/orangebutton',
            frameCountX: 1,
            frameCountY: 3,
            alpha: 1,
            position: settings.position || new Vector2(0, 0),
            scale: Globals.pixelScaleV.multiply(buttonScalar),
            float: true,
            sort: true,
            onClick: function () {
                if (isWatchingAd) {
                    return;
                }
                isWatchingAd = true;
                if (Utils.isDev()) {
                    window.alert("This is a Rewarded Ad!");
                    Bento.input.resetPointers();
                    SaveManager.save('levelsSinceBonusAd', 0);
                    Globals.attemptsSinceAd = 0;
                    giveSuperChest();
                } else {
                    Ads.showRewarded(function () {
                        EventSystem.fire('GameAnalytics-addDesignEvent', {
                            eventId: "ads:superChestMenuAd",
                            value: 1
                        });
                        SaveManager.save('levelsSinceBonusAd', 0);
                        Globals.attemptsSinceAd = 0;
                        giveSuperChest();
                    }, function (e) {
                        isWatchingAd = false;
                    }, "SpinSuperChestMenu");
                }
            }
        });
        button.attach(buttonBehaviour);
        button.attach(motionBehaviour);
        button.attach(contents);
        return button;
    };
});