cordova.define("cordova-plugin-ironsrc.IronSource", function(require, exports, module) {
/**
 * Cordova IronSource Ad Network Plugin. Please make sure you open .xcworkspace file in Xcode rather than .xcodeproj
 * @class 
 * @example
 *  cordova plugin add -d PathToPluginFolder
 *  cd platforms/ios
 *  pod install 
 */

var exec = require('cordova/exec');

/**
 * This function is used to initialize the IronSource SDK with app key. SDK should be initialize first before making any other call.
 *
 * @method initSdk
 * @summary USAGE - window.plugins.ironsource.initSdk(adId, successCallback, errorCallback)
 * @param {string} appKey - App Key.
 * @param {function} successCallback - Callback function which will be called when sdk is initialized successfully.
 * @param {function} errorCallback - Error callback function.
 * @example
 * // object should be passed with all keys even when there is nothing in their values.
 *  var options = {
 *       appKey: '7778606d', //required
 *       userId: '', //optional
 *       clientCallback: true // or flase
 *   };
 *
 * window.plugins.ironsource.initSdk(options, successCallback, errorCallback)
 */
exports.initSdk = function (options, success, error) {
	if (options.hasOwnProperty('appKey') === false) {
		error('appKey not provided');
		return;
	}

	if (options.hasOwnProperty('userId') === false) {
		error('userId not provided');
		return;
	}

	if (options.hasOwnProperty('clientCallback') === false) {
		error('clientCallback not provided');
		return;
	}
	exec(success, error, 'IronSourcePlugin', 'initSdk', [options.appKey, options.userId, options.clientCallback]);
};

/**
 * This function is used to validate the IronsSource SDK and Mediation integration. The log is shown in Xcode or Android Monitor.
 *
 * @method validateIntegration
 * @summary USAGE - window.plugins.ironsource.validateIntegration(successCallback, errorCallback)
 * @param {function} successCallback - Callback function which will be called when interstitial view is created.
 * @param {function} errorCallback - Callback function which will be called if there is an error.
 * @example
 *  window.plugins.ironsource.validateIntegration(successCallback, errorCallback)
 */
exports.validateIntegration = function (success, error) {
	exec(success, error, 'IronSourcePlugin', 'validateIntegration', []);
};
/**
 * This function is used to set the user consent.
 *
 * @method setUserConsent
 * @summary USAGE - window.plugins.ironsource.setUserConsent(userConsent, successCallback, errorCallback)
 * @param {function} successCallback - Callback function which will be called when interstitial view is created.
 * @param {function} errorCallback - Callback function which will be called if there is an error.
 * @example
 *  var userConsent = true; //or false boolean value
 *  window.plugins.ironsource.setUserConsent(userConsent, successCallback, errorCallback)
 */
exports.setUserConsent = function (userConsent, success, error) {
	exec(success, error, 'IronSourcePlugin', 'setUserConsent', [userConsent]);
};
/**
 * This function is used to set the dynamic user id.
 *
 * @method setDynamicUserId
 * @summary USAGE - window.plugins.ironsource.setDynamicUserId(dynamicUserId, successCallback, errorCallback)
 * @param {function} successCallback - Callback function which will be called when interstitial view is created.
 * @param {function} errorCallback - Callback function which will be called if there is an error.
 * @example
 *  var dynamicUserId = ""; //requried string value
 *  window.plugins.ironsource.setDynamicUserId(dynamicUserId, successCallback, errorCallback)
 */
exports.setDynamicUserId = function (userConsent, success, error) {
	exec(success, error, 'IronSourcePlugin', 'setDynamicUserId', [userConsent]);
};

/**
 * This function is used to initialize the IronSource Ad Banner View. It doesnt load, it just prepare the ad and ad view. To load the ad call showBanner method.
 *
 * @method createBannerView
 * @summary USAGE - cwindow.plugins.ironsource.createBannerView(options, successCallback, errorCallback)
 * @param {Object} options - See example for options available.
 * @param {function} successCallback - Callback function which will be called banner view is created.
 * @param {function} errorCallback - Callback function which will be called if there is an error.
 * @example
 *      var options = {
 *             autoShowBanner: 'false', // true or false - To auto load ad when ad is ready.
 *             bannerPosition: 'bottom',// center or top or bottom. Center always overlap the screen.
 *             adSize: 'SMART', // SMART OR BANNER OR LARGE_BANNER or RECT_BANNER
 *             placement: 'Home_Screen', // placement name optional
 *             bannerOverlap: 'true', // true or false, overlap banner on webview. doesn't have effect on center position.
 *             offsetTopBar: 'false', // true or false, set offset when ad is loaded at top for ios7 statusbar
 *         };
 *
 *  window.plugins.ironsource.createBannerView(options, successCallback, errorCallback)
 */
exports.createBannerView = function (options, success, error) {
	exec(success, error, 'IronSourcePlugin', 'createBannerView', [options]);
};
/**
 * This function is used to remove the IronSource Ad Banner sub view.
 *
 * @method removeBannerView
 * @summary USAGE - window.plugins.ironsource.removeBannerView(successCallback, errorCallback)
 * @param {function} successCallback - Callback function which will be called banner view is removed.
 * @param {function} errorCallback - Callback function which will be called if there is an error.
 * @example
 *  window.plugins.ironsource.removeBannerView(successCallback, errorCallback)
 */

exports.removeBannerView = function (success, error) {
	exec(success, error, 'IronSourcePlugin', 'removeBannerView', []);
};
/**
 * This function is used to display the banner ad which is intialized first using the create banner view.
 *
 * @method showBanner
 * @summary USAGE - window.plugins.ironsource.showBanner(successCallback, errorCallback)
 * @example
 *  window.plugins.ironsource.showBanner(successCallback, errorCallback)
 */
exports.showBanner = function (success, error) {
	exec(success, error, 'IronSourcePlugin', 'showBanner', []);
};

/**
 * This function hides the currently displaying ad, but doesn't remove it completely like removeBannerView.
 *
 * @method hideBanner
 * @summary USAGE - window.plugins.ironsource.hideBanner(successCallback, errorCallback)
 * @example
 *  ordova.plugins.ironsource.hideBanner(successCallback, errorCallback)
 */
exports.hideBanner = function (success, error) {
	exec(success, error, 'IronSourcePlugin', 'hideBanner', []);
};

/**
 * This function is used to initialize the Interstitial ads. It doesnt load, it just prepare the ad. To load the ad call showInterstitialView method.
 *
 * @method prepareInterstitialView
 * @summary USAGE - window.plugins.ironsource.prepareInterstitialView(successCallback, errorCallback)
 * @param {function} successCallback - Callback function which will be called when interstitial view is created.
 * @param {function} errorCallback - Callback function which will be called if there is an error.
 * @example
 *
 *  window.plugins.ironsource.prepareInterstitialView(successCallback, errorCallback)
 */

exports.prepareInterstitialView = function (success, error) {
	exec(success, error, 'IronSourcePlugin', 'prepareInterstitialView', []);
};
/**
 * This function is used to check if the interstitial ad is ready to get displayed.
 * Returns true if ready otherwise false
 *
 * @method isInterstitialReady
 * @summary USAGE - window.plugins.ironsource.isInterstitialReady(successCallback, errorCallback)
 * @example
 *    window.plugins.ironsource.isInterstitialReady(successCallback, errorCallback)
 */
exports.isInterstitialReady = function (success, error) {
	exec(success, error, 'IronSourcePlugin', 'isInterstitialReady', []);
};
/**
 * This function is used to display the interstitial ad which is intialized first using the create interstial view method. Please use isInterstialReady method to check if the ad is * ready to get displayed.
 *
 * @method showInterstitialView
 * @summary USAGE - window.plugins.ironsource.showInterstitialView(placement, successCallback, errorCallback)
 * @example
 *  var placment = ''; //optional param
 *  window.plugins.ironsource.showInterstitialView(placement, successCallback, errorCallback)
 */
exports.showInterstitialView = function (placement, success, error) {
	if (!placement) {
		placement = '';
	}
	exec(success, error, 'IronSourcePlugin', 'showInterstitialView', [placement]);
};
/**
 * This function is used to check if the offerwall is present.
 * Returns true if ready otherwise false
 *
 * @method hasOfferwall
 * @summary USAGE - window.plugins.ironsource.hasOfferwall(successCallback, errorCallback)
 * @example
 *    window.plugins.ironsource.hasOfferwall(successCallback, errorCallback)
 */
exports.hasOfferwall = function (success, error) {
	exec(success, error, 'IronSourcePlugin', 'hasOfferwall', []);
};
/**
 * This function is used to display the offerwall. Please use hasOfferwall method to check if the offerwall is present.
 *
 * @method showOfferwall
 * @summary USAGE - window.plugins.ironsource.showOfferwall(placement, successCallback, errorCallback)
 * @example
 *  var placment = ''; //optional param
 *  window.plugins.ironsource.showOfferwall(placement, successCallback, errorCallback)
 */
exports.showOfferwall = function (placement, success, error) {
	if (!placement) {
		placement = '';
	}
	exec(success, error, 'IronSourcePlugin', 'showOfferwall', [placement]);
};
/**
 * This function is used to get information on the user’s total credits and any new credits the user has earned. It doesn't directly returns the data, but offerwall listener, didReceiveOfferwallCredits,is fired when data is available.
 *
 * @method userOfferwallCredits
 * @summary USAGE - window.plugins.ironsource.userOfferwallCredits(successCallback, errorCallback)
 * @example
 *    window.plugins.ironsource.userOfferwallCredits(successCallback, errorCallback)
 */
exports.userOfferwallCredits = function (success, error) {
	exec(success, error, 'IronSourcePlugin', 'userOfferwallCredits', []);
};

/**
 * This function is used to check if the rewarded video is present or not.
 *
 * @method hasRewardedVideo
 * @summary USAGE - window.plugins.ironsource.hasRewardedVideo(successCallback, errorCallback)
 * @example
 *  window.plugins.ironsource.hasRewardedVideo(successCallback, errorCallback)
 */
exports.hasRewardedVideo = function (success, error) {
	exec(success, error, 'IronSourcePlugin', 'hasRewardedVideo', []);
};
/**
 * This function is used to verify if a specific placement has reached its ad limit.
 *
 * @method isRewardedVideoCappedForPlacement
 * @summary USAGE - window.plugins.ironsource.isRewardedVideoCappedForPlacement(placement, successCallback, errorCallback)
 * @example
 *  var placement = "";
 *  window.plugins.ironsource.isRewardedVideoCappedForPlacement(placement, successCallback, errorCallback)
 */
exports.isRewardedVideoCappedForPlacement = function (success, error) {
	exec(success, error, 'IronSourcePlugin', 'isRewardedVideoCappedForPlacement', []);
};
/**
 * This function is used to check if the rewarded video is present or not. It will fire the rewardedVideoHasChangedAvailability listener with the status.
 *
 * @method showRewardedVideo
 * @summary USAGE - window.plugins.ironsource.showRewardedVideo(placement, successCallback, errorCallback)
 * @example
 *  var placement = ""; //optional
 *  window.plugins.ironsource.showRewardedVideo(placement, successCallback, errorCallback)
 */
exports.showRewardedVideo = function (placement, success, error) {
	if (!placement) {
		placement = '';
	}
	exec(success, error, 'IronSourcePlugin', 'showRewardedVideo', [placement]);
};
/**
 * Following listeners are fired for banner ads
 * @event Banner Ad Listener
 * @example
 * var d = function(data){
 *  console.log(data);
 * };
 * document.addEventListener('bannerDidLoad', d, false);
 * document.addEventListener('bannerDidFailToLoadWithError', d, false);
 * document.addEventListener('didClickBanner', d, false);
 * document.addEventListener('bannerWillPresentScreen', d, false);
 * document.addEventListener('bannerDidDismissScreen', d, false);
 * document.addEventListener('bannerWillLeaveApplication', d, false);
 */

/**
 * Following listeners are fired for Interstial ads
 * @event Interstial Ad Listener
 * @example
 * var d = function(data){
 *  console.log(data);
 * };
 * document.addEventListener('interstitialDidLoad', d, false);
 * document.addEventListener('interstitialDidShow', d, false);
 * document.addEventListener('interstitialDidFailToShowWithError', d, false);
 * document.addEventListener('didClickInterstitial', d, false);
 * document.addEventListener('interstitialDidClose', d, false);
 * document.addEventListener('interstitialDidOpen', d, false);
 * document.addEventListener('interstitialDidFailToLoadWithError', d, false);
 */

/**
 * Following listeners are fired for Reward Video ads
 * @event Reward Video Ad Listener
 * @example
 * var d = function(data){
 *  console.log(data);
 * };
 * document.addEventListener('didClickRewardedVideo', d, false);
 * document.addEventListener('rewardedVideoDidEnd', d, false);
 * document.addEventListener('rewardedVideoDidStart', d, false);
 * document.addEventListener('rewardedVideoDidClose', d, false);
 * document.addEventListener('rewardedVideoDidOpen', d, false);
 * document.addEventListener('rewardedVideoDidFailToShowWithError', d, false);
 * document.addEventListener('didReceiveRewardForPlacement', d, false);
 * document.addEventListener('rewardedVideoHasChangedAvailability', d, false);
 */

/**
 * Following listeners are fired for Offerwall
 * @event Offerwall Listener
 * @example
 * var d = function(data){
 *  console.log(data);
 * };
 * document.addEventListener('didReceiveOfferwallCredits', d, false);
 * document.addEventListener('offerwallDidClose', d, false);
 * document.addEventListener('didFailToReceiveOfferwallCreditsWithError', d, false);
 * document.addEventListener('offerwallHasChangedAvailability', d, false);
 * document.addEventListener('offerwallDidShow', d, false);
 * document.addEventListener('offerwallDidFailToShowWithError', d, false);
 */
exports.fireDocumentAdEvent = function (eventName, data) {
	console.log(data);
	var event = document.createEvent('Event');
	event.initEvent(eventName, true, true);
	event.msg = data;
	document.dispatchEvent(event);
};
});
